"use strict";

var camelize = require("./inflector").camelize,
    callOnce = require("./call_once"),
    domAdapter = require("../dom_adapter");

var jsPrefixes = ["", "Webkit", "Moz", "O", "Ms"],
    cssPrefixes = {
    "": "",
    "Webkit": "-webkit-",
    "Moz": "-moz-",
    "O": "-o-",
    "ms": "-ms-"
},
    getStyles = callOnce(function () {
    return domAdapter.createElement("dx").style;
});

var forEachPrefixes = function forEachPrefixes(prop, callBack) {
    prop = camelize(prop, true);

    var result;

    for (var i = 0, cssPrefixesCount = jsPrefixes.length; i < cssPrefixesCount; i++) {
        var jsPrefix = jsPrefixes[i];
        var prefixedProp = jsPrefix + prop;
        var lowerPrefixedProp = camelize(prefixedProp);

        result = callBack(lowerPrefixedProp, jsPrefix);

        if (result === undefined) {
            result = callBack(prefixedProp, jsPrefix);
        }

        if (result !== undefined) {
            break;
        }
    }

    return result;
};

var styleProp = function styleProp(name) {
    if (name in getStyles()) {
        return name;
    }

    name = name.charAt(0).toUpperCase() + name.substr(1);
    for (var i = 1; i < jsPrefixes.length; i++) {
        var prefixedProp = jsPrefixes[i].toLowerCase() + name;
        if (prefixedProp in getStyles()) {
            return prefixedProp;
        }
    }
};

var stylePropPrefix = function stylePropPrefix(prop) {
    return forEachPrefixes(prop, function (specific, jsPrefix) {
        if (specific in getStyles()) {
            return cssPrefixes[jsPrefix];
        }
    });
};

exports.styleProp = styleProp;
exports.stylePropPrefix = stylePropPrefix;