"use strict";

var $ = require("../../core/renderer"),
    extend = require("../../core/utils/extend").extend,
    getDefaultAlignment = require("../../core/utils/position").getDefaultAlignment,
    commonUtils = require("../../core/utils/common");

var COLUMN_INDICATORS_CLASS = "dx-column-indicators",
    GROUP_PANEL_ITEM_CLASS = "dx-group-panel-item";

module.exports = {
    _applyColumnState: function _applyColumnState(options) {
        var that = this,
            rtlEnabled = this.option("rtlEnabled"),
            columnAlignment = that._getColumnAlignment(options.column.alignment, rtlEnabled),
            parameters = extend(true, { columnAlignment: columnAlignment }, options),
            isGroupPanelItem = parameters.rootElement.hasClass(GROUP_PANEL_ITEM_CLASS),
            $indicatorsContainer = that._createIndicatorContainer(parameters, isGroupPanelItem),
            $span = $("<span>").addClass(that._getIndicatorClassName(options.name)),
            getIndicatorAlignment = function getIndicatorAlignment() {
            if (rtlEnabled) {
                return columnAlignment === "left" ? "right" : "left";
            }
            return columnAlignment;
        };

        parameters.container = $indicatorsContainer;
        parameters.indicator = $span;
        that._renderIndicator(parameters);

        $indicatorsContainer[(isGroupPanelItem || !options.showColumnLines) && getIndicatorAlignment() === "left" ? "appendTo" : "prependTo"](options.rootElement);

        return $span;
    },

    _getIndicatorClassName: commonUtils.noop,

    _getColumnAlignment: function _getColumnAlignment(alignment, rtlEnabled) {
        rtlEnabled = rtlEnabled || this.option("rtlEnabled");

        return alignment && alignment !== "center" ? alignment : getDefaultAlignment(rtlEnabled);
    },

    _createIndicatorContainer: function _createIndicatorContainer(options, ignoreIndicatorAlignment) {
        var $indicatorsContainer = this._getIndicatorContainer(options.rootElement),
            indicatorAlignment = options.columnAlignment === "left" ? "right" : "left";

        if (!$indicatorsContainer.length) {
            $indicatorsContainer = $("<div>").addClass(COLUMN_INDICATORS_CLASS);
        }

        return $indicatorsContainer.css("float", options.showColumnLines && !ignoreIndicatorAlignment ? indicatorAlignment : null);
    },

    _getIndicatorContainer: function _getIndicatorContainer($cell) {
        return $cell && $cell.find("." + COLUMN_INDICATORS_CLASS);
    },

    _getIndicatorElements: function _getIndicatorElements($cell) {
        var $indicatorContainer = this._getIndicatorContainer($cell);

        return $indicatorContainer && $indicatorContainer.children();
    },

    _renderIndicator: function _renderIndicator(options) {
        var $container = options.container,
            $indicator = options.indicator;

        $container && $indicator && $container.append($indicator);
    },

    _updateIndicators: function _updateIndicators(indicatorName) {
        var that = this,
            columns = that.getColumns(),
            $cells = that.getColumnElements(),
            rowOptions,
            $cell,
            i;

        if (!$cells || !$cells.length) return;

        for (i = 0; i < columns.length; i++) {
            $cell = $cells.eq(i);
            that._updateIndicator($cell, columns[i], indicatorName);

            rowOptions = $cell.parent().data("options");

            if (rowOptions && rowOptions.cells) {
                rowOptions.cells[$cell.index()].column = columns[i];
            }
        }
    },

    _updateIndicator: function _updateIndicator($cell, column, indicatorName) {
        if (!column.command) {
            return this._applyColumnState({
                name: indicatorName,
                rootElement: $cell,
                column: column,
                showColumnLines: this.option("showColumnLines")
            });
        }
    }
};